@@ -57,7 +57,7 @@
 #' @param shape geometric shape used in the diagram
 #' @param control a list of control parameters.
 #'   * `extraopt`: should the more thorough optimizer (currently
-#'   [RcppDE::DEoptim()]) kick in (provided `extraopt_threshold` is exceeded)? The
+#'   [GenSA::GenSA()]) kick in (provided `extraopt_threshold` is exceeded)? The
 #'   default is `TRUE` for ellipses and three sets and `FALSE` otherwise.
 #'   * `extraopt_threshold`: threshold, in terms of `diagError`, for when
 #'   the extra optimizer kicks in. This will almost always slow down the
@@ -65,7 +65,7 @@
 #'   that the extra optimizer will kick in if there is *any* error. A value of
 #'   1 means that it will never kick in. The default is `0.001`.
 #'   * `extraopt_control`: a list of control parameters to pass to the
-#'   extra optimizer, such as `itermax`. See [RcppDE::DEoptim.control()].
+#'   extra optimizer, such as `max.call`. See [GenSA::GenSA()].
 #' @param ... arguments passed down to other methods
 #'
 #' @return A list object of class `'euler'` with the following parameters.
@@ -137,6 +137,8 @@ euler.default <- function(
   combo_names <- strsplit(names(combinations), split = "&", fixed = TRUE)
   setnames <- unique(unlist(combo_names, use.names = FALSE))
 
+  # setup preliminary return pars
+
   n <- length(setnames)
   id <- bit_indexr(n)
   N <- NROW(id)
@@ -159,213 +161,224 @@ euler.default <- function(
     }
   }
 
+  # Decompose or collect set volumes depending on input
+  if (match.arg(input) == "disjoint") {
+    areas_disjoint <- areas
+    areas[] <- 0
+    for (i in rev(seq_along(areas))) {
+      prev_areas <- rowSums(id[, id[i, ], drop = FALSE]) == sum(id[i, ])
+      areas[i] <- sum(areas_disjoint[prev_areas])
+    }
+  } else if (match.arg(input) == "union") {
+    areas_disjoint <- double(length(areas))
+    for (i in rev(seq_along(areas))) {
+      prev_areas <- rowSums(id[, id[i, ], drop = FALSE]) == sum(id[i, ])
+      areas_disjoint[i] <- areas[i] - sum(areas_disjoint[prev_areas])
+    }
+    if (any(areas_disjoint < 0))
+      stop("Check your set configuration. Some disjoint areas are negative.")
+  }
+
+  # setup return object
+  fpar <- as.data.frame(matrix(
+    NA,
+    ncol = 5L,
+    nrow = n,
+    dimnames = list(setnames, c("h", "k", "a", "b", "phi"))
+  ))
+
+  # setup return values
+  orig <- rep.int(0, N)
+  fit <- rep.int(0, N)
+  names(orig) <- names(fit) <-
+    apply(id, 1L, function(x) paste0(setnames[x], collapse = "&"))
+
+  # find empty sets
+  empty_sets <- areas[seq_len(n)] < sqrt(.Machine$double.eps)
+  empty_subsets <- rowSums(id[, empty_sets, drop = FALSE]) > 0
+
+  id <- id[!empty_subsets, !empty_sets, drop = FALSE]
+  N <- NROW(id)
+  n <- sum(!empty_sets)
+  areas <- areas[!empty_subsets]
+  areas_disjoint <- areas_disjoint[!empty_subsets]
+
   if (n > 1L) {
-    # Decompose or collect set volumes depending on input
-    if (match.arg(input) == "disjoint") {
-      areas_disjoint <- areas
-      areas[] <- 0
-      for (i in rev(seq_along(areas))) {
-        prev_areas <- rowSums(id[, id[i, ], drop = FALSE]) == sum(id[i, ])
-        areas[i] <- sum(areas_disjoint[prev_areas])
-      }
-    } else if (match.arg(input) == "union") {
-      areas_disjoint <- double(length(areas))
-      for (i in rev(seq_along(areas))) {
-        prev_areas <- rowSums(id[, id[i, ], drop = FALSE]) == sum(id[i, ])
-        areas_disjoint[i] <- areas[i] - sum(areas_disjoint[prev_areas])
-      }
-      if (any(areas_disjoint < 0))
-        stop("Check your set configuration. Your specification resulted in some disjoint areas being set to 0.")
-    }
-
-    id_sums <- rowSums(id)
-    ones <- id_sums == 1L
-    twos <- id_sums == 2L
-    two <- choose_two(1L:n)
-    r <- sqrt(areas[ones]/pi)
-
-    # Establish identities of disjoint and subset sets
-    subset <- disjoint <- matrix(FALSE, ncol = n, nrow = n)
-    distances <- area_mat <- matrix(0, ncol = n, nrow = n)
-
-    lwrtri <- lower.tri(subset)
-
-    tmp <- matrix(areas[ones][two], ncol = 2L)
-
-    subset[lwrtri] <- areas[twos] == tmp[, 1L] | areas[twos] == tmp[, 2L]
-    disjoint[lwrtri] <- areas[twos] == 0
-    distances[lwrtri] <- mapply(separate_two_discs,
-                                r1 = r[two[, 1L]],
-                                r2 = r[two[, 2L]],
-                                overlap = areas[twos],
-                                USE.NAMES = FALSE)
-
-    # Starting layout
-    loss <- Inf
-    initial_layouts <- vector("list", n_restarts)
-    bnd <- sqrt(sum(r^2*pi))
-
-    i <- 1L
-    while (loss > small && i <= n_restarts) {
-      initial_layouts[[i]] <- stats::nlm(
-        f = optim_init,
-        p = stats::runif(n*2, 0, bnd),
-        d = distances,
-        disjoint = disjoint,
-        subset = subset,
-        iterlim = 1000L
-      )
-      loss <- initial_layouts[[i]]$minimum
-      i <- i + 1L
-    }
-
-    # Find the best initial layout
-    best_init <- which.min(lapply(initial_layouts[1L:(i - 1L)],
-                                  "[[",
-                                  "minimum"))
-    initial_layout <- initial_layouts[[best_init]]
-
-    # Final layout
-    circle <- match.arg(shape) == "circle"
-
-    if (circle) {
-      pars <- as.vector(matrix(c(initial_layout$estimate, r), 3L, byrow = TRUE))
-      lwr <- rep.int(0, 3L)
-      upr <- rep.int(bnd, 3L)
+    if (all(areas == 0)) {
+      # all sets are zero
+      fpar[] <- NA
     } else {
-      pars <- as.vector(rbind(matrix(initial_layout$estimate, 2L, byrow = TRUE),
-                              r, r, 0, deparse.level = 0L))
-      lwr <- c(rep.int(0, 4L), -2*pi)
-      upr <- c(rep.int(bnd, 4L), 2*pi)
-    }
+      id_sums <- rowSums(id)
+      ones <- id_sums == 1L
+      twos <- id_sums == 2L
+      two <- choose_two(1L:n)
+      r <- sqrt(areas[ones]/pi)
 
-    orig <- areas_disjoint
+      # Establish identities of disjoint and subset sets
+      subset <- disjoint <- matrix(FALSE, ncol = n, nrow = n)
+      distances <- area_mat <- matrix(0, ncol = n, nrow = n)
 
-    # Try to find a solution using nlm() first (faster)
-    # TODO: Allow user options here?
-    nlm_solution <- stats::nlm(
-      f = optim_final_loss,
-      p = pars,
-      areas = areas_disjoint,
-      circle = circle,
-      iterlim = 1e4L
-    )$estimate
+      lwrtri <- lower.tri(subset)
 
-    tpar <- as.data.frame(matrix(
-      data = nlm_solution,
-      ncol = if (circle) 3L else 5L,
-      dimnames = list(
-        setnames,
-        if (circle) c("h", "k", "r") else c("h", "k", "a", "b", "phi")
-      ),
-      byrow = TRUE
-    ))
-    if (circle)
-      tpar <- cbind(tpar, tpar[, 3L], 0)
+      tmp <- matrix(areas[ones][two], ncol = 2L)
 
-    # Normalize layout
-    nlm_fit <- as.vector(intersect_ellipses(nlm_solution, circle))
+      subset[lwrtri] <- areas[twos] == tmp[, 1L] | areas[twos] == tmp[, 2L]
+      disjoint[lwrtri] <- areas[twos] == 0
+      distances[lwrtri] <- mapply(separate_two_discs,
+                                  r1 = r[two[, 1L]],
+                                  r2 = r[two[, 2L]],
+                                  overlap = areas[twos],
+                                  USE.NAMES = FALSE)
 
-    nlm_pars <- compress_layout(normalize_pars(tpar), id, nlm_fit)
+      # Starting layout
+      loss <- Inf
+      initial_layouts <- vector("list", n_restarts)
+      bnd <- sqrt(sum(r^2*pi))
 
-    nlm_diagError <- diagError(nlm_fit, orig)
+      i <- 1L
+      while (loss > small && i <= n_restarts) {
+        initial_layouts[[i]] <- stats::nlm(
+          f = optim_init,
+          p = stats::runif(n*2, 0, bnd),
+          d = distances,
+          disjoint = disjoint,
+          subset = subset,
+          iterlim = 1000L
+        )
+        loss <- initial_layouts[[i]]$minimum
+        i <- i + 1L
+      }
 
-    # If inadequate solution, try with a second optimizer (slower, better)
-    if (!circle && control$extraopt &&
-        nlm_diagError > control$extraopt_threshold) {
-      # Set bounds for the parameters
-      newpars <- matrix(
-        data = as.vector(t(nlm_pars)),
-        ncol = 5L,
-        dimnames = list(setnames, c("h", "k", "a", "b", "phi")),
-        byrow = TRUE
-      )
+      # Find the best initial layout
+      best_init <- which.min(lapply(initial_layouts[1L:(i - 1L)],
+                                    "[[",
+                                    "minimum"))
+      initial_layout <- initial_layouts[[best_init]]
 
-      constraints <- get_constraints(compress_layout(newpars, id, nlm_fit))
+      # Final layout
+      circle <- match.arg(shape) == "circle"
 
-      # TODO: Set up initial population in some clever fashion.
+      if (circle) {
+        pars <- as.vector(matrix(c(initial_layout$estimate, r), 3L,
+                                 byrow = TRUE))
+        lwr <- rep.int(0, 3L)
+        upr <- rep.int(bnd, 3L)
+      } else {
+        pars <- as.vector(rbind(matrix(initial_layout$estimate, 2L,
+                                       byrow = TRUE),
+                                r, r, 0, deparse.level = 0L))
+        lwr <- c(rep.int(0, 4L), -2*pi)
+        upr <- c(rep.int(bnd, 4L), 2*pi)
+      }
 
-      deoptim <- RcppDE::DEoptim(
-        fn = optim_final_loss,
-        lower = constraints$lwr,
-        upper = constraints$upr,
-        control = do.call(
-          RcppDE::DEoptim.control,
-          utils::modifyList(
-            list(VTR = -Inf,
-                 NP = length(newpars)*10,
-                 CR = 0.6,
-                 F = 0.2,
-                 itermax = 1000L,
-                 trace = FALSE),
-            control$extraopt_control
-          )
-        ),
-        areas = areas_disjoint,
-        circle = circle
-      )
+      orig[!empty_subsets] <- areas_disjoint
 
-      # Fine tune the fit from DEoptim
-      last_ditch_effort <- stats::nlm(
+      # Try to find a solution using nlm() first (faster)
+      # TODO: Allow user options here?
+      nlm_solution <- stats::nlm(
         f = optim_final_loss,
-        p = deoptim$optim$bestmem,
+        p = pars,
         areas = areas_disjoint,
         circle = circle,
-        iterlim = 1e4L
+        iterlim = 1e6L
       )$estimate
 
-      last_ditch_fit <- as.vector(intersect_ellipses(last_ditch_effort, circle))
-      last_ditch_diagError <- diagError(last_ditch_fit, orig)
+      tpar <- as.data.frame(matrix(
+        data = nlm_solution,
+        ncol = if (circle) 3L else 5L,
+        dimnames = list(
+          setnames[!empty_sets],
+          if (circle) c("h", "k", "r") else c("h", "k", "a", "b", "phi")
+        ),
+        byrow = TRUE
+      ))
+      if (circle)
+        tpar <- cbind(tpar, tpar[, 3L], 0)
 
-      # Check for the best solution
-      if (last_ditch_diagError < nlm_diagError) {
-        final_par <- last_ditch_effort
-        fit <- last_ditch_fit
+      # Normalize layout
+      nlm_fit <- as.vector(intersect_ellipses(nlm_solution, circle))
+
+      nlm_pars <- compress_layout(normalize_pars(tpar), id, nlm_fit)
+
+      nlm_diagError <- diagError(nlm_fit, orig[!empty_subsets])
+
+      # If inadequate solution, try with a second optimizer (slower, better)
+      if (!circle && control$extraopt &&
+          nlm_diagError > control$extraopt_threshold) {
+        # Set bounds for the parameters
+        newpars <- matrix(
+          data = as.vector(t(nlm_pars)),
+          ncol = 5L,
+          dimnames = list(setnames, c("h", "k", "a", "b", "phi")),
+          byrow = TRUE
+        )
+
+        constraints <- get_constraints(compress_layout(newpars, id, nlm_fit))
+
+        last_ditch_effort <- GenSA::GenSA(
+          par = as.vector(newpars),
+          fn = optim_final_loss,
+          lower = constraints$lwr,
+          upper = constraints$upr,
+          circle = circle,
+          areas = areas_disjoint,
+          control = utils::modifyList(
+            list(threshold.stop = sqrt(.Machine$double.eps),
+                 max.call = 1e7,
+                 trace.mat = FALSE),
+            control$extraopt_control
+          )
+        )$par
+
+        last_ditch_fit <- as.vector(intersect_ellipses(last_ditch_effort,
+                                                       circle))
+        last_ditch_diagError <- diagError(last_ditch_fit, orig)
+
+        # Check for the best solution
+        if (last_ditch_diagError < nlm_diagError) {
+          final_par <- last_ditch_effort
+          fit[!empty_subsets] <- last_ditch_fit
+        } else {
+          final_par <- nlm_solution
+          fit[!empty_subsets] <- nlm_fit
+        }
       } else {
         final_par <- nlm_solution
-        fit <- nlm_fit
+        fit[!empty_subsets] <- nlm_fit
       }
-    } else {
-      final_par <- nlm_solution
-      fit <- nlm_fit
+
+      # names(orig) <- names(fit) <-
+      #   apply(id, 1L, function(x) paste0(setnames[x], collapse = "&"))
+
+      regionError <- regionError(fit, orig)
+      diagError <- diagError(regionError = regionError)
+      stress <- stress(orig, fit)
+
+      temp <- matrix(data = final_par,
+                     ncol = if (circle) 3L else 5L,
+                     byrow = TRUE)
+
+      if (circle)
+        temp <- cbind(temp, temp[, 3L], 0)
+
+      # Normalize semiaxes and rotation
+      temp <- normalize_pars(temp)
+
+      # Find disjoint clusters and compress the layout
+      temp <- compress_layout(temp, id, fit[!empty_subsets])
+
+      # Center the solution on the coordinate plane
+      fpar[!empty_sets, ] <- center_layout(temp)
     }
-
-    names(orig) <- names(fit) <-
-      apply(id, 1L, function(x) paste0(setnames[x], collapse = "&"))
-
-    regionError <- regionError(fit, orig)
-    diagError <- diagError(regionError = regionError)
-    stress <- stress(orig, fit)
-
-    fpar <- matrix(data = final_par,
-                   ncol = if (circle) 3L else 5L,
-                   byrow = TRUE)
-
-    if (circle)
-      fpar <- cbind(fpar, fpar[, 3L], 0)
-
-    dimnames(fpar) <- list(setnames, c("h", "k", "a", "b", "phi"))
-
-    # Normalize semiaxes and rotation
-    fpar <- normalize_pars(fpar)
-
-    # Find disjoint clusters and compress the layout
-    fpar <- compress_layout(fpar, id, fit)
-
-    # Center the solution on the coordinate plane
-    fpar <- center_layout(fpar)
   } else {
     # One set
-    fpar <- matrix(data = c(0, 0, sqrt(areas/pi), sqrt(areas/pi), 0),
-                   ncol = 5L,
-                   dimnames = list(setnames, c("h", "k", "a", "b", "phi")))
+    fpar[!empty_sets, ] <- c(0, 0, sqrt(areas/pi), sqrt(areas/pi), 0)
     regionError <- diagError <- stress <- 0
-    orig <- fit <- areas
-    names(orig) <- names(fit) <- setnames
+    orig[!empty_subsets] <- fit[!empty_subsets] <- areas
   }
 
   # Return eulerr structure
-  structure(list(ellipses = as.data.frame(fpar),
+  structure(list(ellipses = fpar,
                  original.values = orig,
                  fitted.values = fit,
                  residuals = orig - fit,
@@ -457,7 +470,7 @@ euler.data.frame <- function(combinations, weights = NULL, by = NULL, ...) {
     colnames(out) <- colnames(combinations)
 
     for (i in seq_along(combinations)) {
-      y <- combinations[, i]
+      y <- combinations[, i, drop = TRUE]
       if (is.factor(y) || is.character(y)) {
         facs <- unique(as.character(y))
         if (length(facs) > 2L)
