@@ -40,14 +40,14 @@
 #'   col        \tab       \tab x     \tab x      \tab x          \tab x      \tab x      \cr
 #'   fill       \tab x     \tab       \tab        \tab            \tab        \tab        \cr
 #'   alpha      \tab x     \tab x     \tab x      \tab x          \tab x      \tab x      \cr
-#'   lty        \tab       \tab       \tab        \tab            \tab        \tab        \cr
-#'   lwd        \tab       \tab       \tab        \tab            \tab        \tab        \cr
-#'   lex        \tab       \tab       \tab        \tab            \tab        \tab        \cr
-#'   fontsize   \tab       \tab x     \tab x      \tab x          \tab x      \tab x      \cr
-#'   cex        \tab       \tab x     \tab x      \tab x          \tab x      \tab x      \cr
-#'   fontfamily \tab       \tab x     \tab x      \tab x          \tab x      \tab x      \cr
-#'   lineheight \tab       \tab x     \tab x      \tab x          \tab x      \tab x      \cr
-#'   font       \tab       \tab x     \tab x      \tab x          \tab x      \tab x
+#'   lty        \tab       \tab x     \tab        \tab            \tab        \tab        \cr
+#'   lwd        \tab       \tab x     \tab        \tab            \tab        \tab        \cr
+#'   lex        \tab       \tab x     \tab        \tab            \tab        \tab        \cr
+#'   fontsize   \tab       \tab       \tab x      \tab x          \tab x      \tab x      \cr
+#'   cex        \tab       \tab       \tab x      \tab x          \tab x      \tab x      \cr
+#'   fontfamily \tab       \tab       \tab x      \tab x          \tab x      \tab x      \cr
+#'   lineheight \tab       \tab       \tab x      \tab x          \tab x      \tab x      \cr
+#'   font       \tab       \tab       \tab x      \tab x          \tab x      \tab x
 #' }
 #'
 #' Defaults for these values, as well as other parameters of the plots, can
@@ -266,6 +266,7 @@ plot.euler <- function(x,
                      rot = opar$labels$rot)
     }
 
+    labels$rot <- rep_len(labels$rot, n_e)
     labels$gp <- setup_gpar(list(col = opar$labels$col,
                                  alpha = opar$labels$alpha,
                                  fontsize = opar$labels$fontsize,
@@ -290,6 +291,7 @@ plot.euler <- function(x,
     } else {
       quantities <- list(labels = quantities, rot = opar$quantities$rot)
     }
+    quantities$rot <- rep_len(quantities$rot, n_id)
 
     quantities$gp <- setup_gpar(list(col = opar$quantities$col,
                                      alpha = opar$quantities$alpha,
@@ -299,7 +301,7 @@ plot.euler <- function(x,
                                      lineheight = opar$quantities$lineheight,
                                      font = opar$quantities$font),
                                 quantities,
-                                n_e)
+                                n_id)
   } else {
     quantities <- NULL
   }
@@ -346,6 +348,8 @@ plot.euler <- function(x,
     legend <- NULL
   }
 
+  # set up geometry for diagrams
+
   if (do_groups) {
     data <- lapply(x,
                    setup_geometry,
@@ -365,189 +369,23 @@ plot.euler <- function(x,
                            id)
   }
 
-  structure(list(fills = fills,
-                 edges = edges,
-                 labels = labels,
-                 quantities = quantities,
-                 strips = strips,
-                 legend = legend,
-                 data = data),
-            class = c("eulergram"))
-}
-
-#' Compute geometries and label locations
-#'
-#' @param x an object of class 'euler'
-#' @param do_fills do fills?
-#' @param do_edges do edges?
-#' @param do_labels do labels?
-#' @param do_quantities do quantities?
-#'
-#' @return a list object with slots for the various objects
-#' @keywords internal
-setup_geometry <- function(x,
-                           fills,
-                           edges,
-                           labels,
-                           quantities,
-                           n,
-                           id) {
-  dd <- x$ellipses
-  orig <- x$original.values
-  fitted <- x$fitted.values
-
-  do_fills <- !is.null(fills)
-  do_edges <- !is.null(edges)
-  do_labels <- !is.null(labels)
-  do_quantities <- !is.null(quantities)
-
-  h <- dd$h
-  k <- dd$k
-  a <- dd$a
-  b <- dd$b
-  phi <- dd$phi
-
-  n_e <- NROW(dd)
-  n_id <- 2L^n_e - 1L
-
-  e <- ellipse(h, k, a, b, phi, n)
-  e_x <- c(lapply(e, "[[", "x"), recursive = TRUE)
-  e_y <- c(lapply(e, "[[", "y"), recursive = TRUE)
-
-  limits <- get_bounding_box(h, k, a, b, phi)
-
-  # setup edges
-  if (do_edges)
-    edges <- list(x = e_x, y = e_y, id.lengths = rep.int(n, n_e))
-
-  if (do_fills) {
-    # overlay ellipses on top of each other
-    pieces <- fills <- vector("list", n_id)
-    for (i in rev(seq_len(n_id))) {
-      idx <- which(id[i, ])
-      n_idx <- length(idx)
-      if (n_idx == 1L) {
-        pieces[[i]] <- e[[idx[1]]]
-      } else {
-        pieces[[i]] <- poly_clip(e[[idx[1L]]], e[[idx[2L]]], "intersection")
-        if (n_idx > 2L) {
-          for (j in 3L:n_idx) {
-            pieces[[i]] <- poly_clip(pieces[[i]], e[[idx[j]]], "intersection")
-          }
-        }
-      }
-      for (ii in which(!id[i, ])) {
-        pieces[[i]] <- poly_clip(pieces[[i]], e[[ii]], "minus")
-      }
-
-      for (i in seq_along(pieces)) {
-        if (is.null(pieces[[i]]$x)) {
-          x0 <- lapply(pieces[[i]], "[[", "x")
-          y0 <- lapply(pieces[[i]], "[[", "y")
-        } else {
-          x0 <- pieces[[i]]$x
-          y0 <- pieces[[i]]$y
-        }
-        if (length(x0) > 0L) {
-          fills[[i]]$x <- c(x0, recursive = TRUE)
-          fills[[i]]$y <- c(y0, recursive = TRUE)
-          fills[[i]]$id.lengths <- lengths(x0)
-        }
-      }
-    }
-  }
-
-  if (do_labels || do_quantities) {
-    singles <- rowSums(id) == 1
-    empty <- abs(fitted) < sqrt(.Machine$double.eps)
-
-    centers <- cbind(t(locate_centers(h, k, a, b, phi, fitted)), seq_len(n_id))
-    rownames(centers) <- names(orig)
-
-    if (do_labels) {
-      labels <- list(labels = labels$labels)
-      center_labels <- labels$labels[!is.nan(centers[singles, 1L])]
-    }
-
-    labels_centers <- centers[!is.nan(centers[, 1L]) & singles, , drop = FALSE]
-
-    droprows <- rep.int(TRUE, NROW(centers))
-    for (i in which(is.nan(centers[singles, 1L]))) {
-      pick <- id[, i] & !empty & !is.nan(centers[, 1L])
-      labels_centers <- rbind(labels_centers, centers[which(pick)[1L], ])
-      if (do_labels)
-        center_labels <- c(center_labels, labels$labels[i])
-      droprows[which(pick)[1L]] <- FALSE
-    }
-
-    if (do_quantities) {
-      quantities_centers <-
-        centers[!is.nan(centers[, 1L]) & !singles & droprows, , drop = FALSE]
-      quantities_centers <- rbind(quantities_centers, labels_centers)
-      if (!is.null(quantities$labels))
-        quantities <- list(labels = quantities$labels[quantities_centers[, 3L]])
-      else
-        quantities <- list(labels = orig[quantities_centers[, 3L]])
-      quantities$x <- quantities_centers[, 1L]
-      quantities$y <- quantities_centers[, 2L]
-    }
-
-    if (do_labels) {
-      labels$x <- labels_centers[, 1L]
-      labels$y <- labels_centers[, 2L]
-      labels$labels <- center_labels
-    }
-  }
-
-  list(ellipses = dd,
-       fitted.values = fitted,
-       original.values = orig,
-       fills = fills,
-       edges = edges,
-       labels = labels,
-       quantities = quantities,
-       xlim = limits$xlim,
-       ylim = limits$ylim)
-}
-
-#' Print (plot) Euler diagram
-#'
-#' This function is responsible for the actual drawing of
-#' `'eulergram'` objects created through [plot.euler()]. [print.eulergram()]
-#' is an alias for [plot.eulergram()], which has been provided so that
-#' [plot.euler()] gets called automatically.
-#'
-#' @param x an object of class `'eulergram'`, usually the output of
-#'   [plot.euler()]
-#' @param newpage if `TRUE`, opens a new page via [grid.newpage()] to draw on
-#' @param ... ignored
-#' @return A plot is drawn on the current device using [grid::Grid()] graphics.
-#' @export
-plot.eulergram <- function(x, newpage = TRUE, ...) {
-  legend <- x$legend
-  fills <- x$fills
-  edges <- x$edges
-  labels <- x$labels
-  quantities <- x$quantities
-  data <- x$data
-  strips <- x$strips
   groups <- strips$groups
 
-  opar <- eulerr_options()
-
-  do_legend <- !is.null(legend)
-  do_groups <- !is.null(strips)
+  # start setting up grobs
 
   if (do_groups) {
-    euler_grob <- lapply(
-      data,
-      setup_grobs,
-      fills = fills,
-      edges = edges,
-      labels = labels,
-      quantities = quantities
-    )
-    euler_grob <- do.call(grid::gList, euler_grob)
+    n_groups <- length(data)
+    euler_grob_children <- grid::gList()
+    for (i in seq_len(n_groups)) {
+      euler_grob_children[[i]] <- setup_grobs(data[[i]],
+                                              fills = fills,
+                                              edges = edges,
+                                              labels = labels,
+                                              quantities = quantities,
+                                              number = i)
+    }
+    euler_grob <- grid::gTree(grid::nullGrob(),
+                              children = euler_grob_children)
     pos <- vapply(groups, as.numeric, numeric(NROW(groups)), USE.NAMES = FALSE)
     layout <- lengths(lapply(groups, unique))
     if (length(layout) == 1L)
@@ -559,8 +397,9 @@ plot.eulergram <- function(x, newpage = TRUE, ...) {
                               fills = fills,
                               edges = edges,
                               labels = labels,
-                              quantities = quantities)
-    euler_grob <- grid::gList(euler_grob)
+                              quantities = quantities,
+                              number = 1)
+    euler_grob <- grid::grobTree(euler_grob)
     xlim <- data$xlim
     ylim <- data$ylim
     pos <- c(1L, 1L)
@@ -571,6 +410,12 @@ plot.eulergram <- function(x, newpage = TRUE, ...) {
   ylim <- grDevices::extendrange(ylim, f = 0.01)
   xrng <- abs(xlim[1L] - xlim[2L])
   yrng <- abs(ylim[1L] - ylim[2L])
+
+  if (xrng == 0 || yrng == 0) {
+    xrng <- yrng <- 1
+    ylim <- xlim <- c(0, 1)
+  }
+
   ar <- xrng/yrng
   adjust <- layout[1L]/layout[2]
 
@@ -663,62 +508,40 @@ plot.eulergram <- function(x, newpage = TRUE, ...) {
                                          c("lines", "grobheight"),
                                          list(NULL, legend_grob)))
     }
-  } else {
-    do_legend <- FALSE
+    legend_grob$vp <- grid::viewport(layout.pos.row = legend_row,
+                                     layout.pos.col = legend_col,
+                                     name = "legend.vp")
   }
 
-  parent <- grid::viewport(layout = grid::grid.layout(nrow = nrow,
-                                                      ncol = ncol,
-                                                      widths = widths,
-                                                      heights = heights,
-                                                      respect = TRUE),
-                           name = "euler.vp",
-                           gp = grid::gpar(fontsize = opar$fontsize))
-
-  panel_layout <- grid::grid.layout(nrow = layout[1L],
-                                    ncol = layout[2L],
-                                    widths = rep(1/ar, layout[1L]),
-                                    heights = rep(1, layout[2L]))
-
-  children <- grid::vpList(grid::viewport(layout.pos.row = diagram_row,
-                                          layout.pos.col = diagram_col,
-                                          name = "canvas.vp",
-                                          layout = panel_layout))
-  k <- 2L
-
-  if (do_legend) {
-    children[[k]] <- grid::viewport(layout.pos.row = legend_row,
-                                    layout.pos.col = legend_col,
-                                    name = "legend.vp")
-    k <- k + 1L
-  }
+  canvas_vp <- grid::viewport(
+    layout.pos.row = diagram_row,
+    layout.pos.col = diagram_col,
+    name = "canvas.vp",
+    layout = grid::grid.layout(nrow = layout[1L],
+                               ncol = layout[2L],
+                               widths = rep(1/ar, layout[1L]),
+                               heights = rep(1, layout[2L]))
+  )
 
   if (do_strip_top) {
-    strip_top_layout <- grid::grid.layout(nrow = 1L, ncol = layout[2L])
-    children[[k]] <- grid::viewport(layout.pos.row = strip_top_row,
-                                    layout.pos.col = strip_top_col,
-                                    name = "strip.top.vp",
-                                    layout = strip_top_layout)
-    k <- k + 1L
+    strip_top_vp <- grid::viewport(
+      layout.pos.row = strip_top_row,
+      layout.pos.col = strip_top_col,
+      name = "strip.top.vp",
+      layout = grid::grid.layout(nrow = 1L, ncol = layout[2L])
+    )
   }
 
   if (do_strip_left) {
-    strip_left_layout <- grid::grid.layout(nrow = layout[1L], ncol = 1L)
-    children[[k]] <- grid::viewport(layout.pos.row = strip_left_row,
-                                    layout.pos.col = strip_left_col,
-                                    name = "strip.left.vp",
-                                    layout = strip_left_layout)
-    k <- k + 1L
+    strip_left_vp <- grid::viewport(
+      layout.pos.row = strip_left_row,
+      layout.pos.col = strip_left_col,
+      name = "strip.left.vp",
+      layout = grid::grid.layout(nrow = layout[1L], ncol = 1L)
+    )
   }
 
-  if (newpage)
-    grid::grid.newpage()
-  grid::pushViewport(grid::vpTree(parent, children))
-  grid::upViewport()
-
-  # draw diagram
-  grid::downViewport("canvas.vp")
-  for (i in seq_along(euler_grob)) {
+  for (i in seq_along(euler_grob$children)) {
     if (NCOL(pos) == 2L) {
       j <- pos[i, 1L]
       k <- pos[i, 2L]
@@ -726,83 +549,255 @@ plot.eulergram <- function(x, newpage = TRUE, ...) {
       j <- 1L
       k <- pos[i]
     }
-    grid::pushViewport(grid::viewport(layout.pos.row = j,
-                                      layout.pos.col = k,
-                                      xscale = xlim,
-                                      yscale = ylim,
-                                      name = paste0("panel.vp.", j, ".", k)))
-    grid::grid.draw(euler_grob[[i]])
-    grid::upViewport()
+    euler_grob$children[[i]]$vp <- grid::viewport(
+      layout.pos.row = j,
+      layout.pos.col = k,
+      xscale = xlim,
+      yscale = ylim,
+      name = paste0("panel.vp.", j, ".", k)
+    )
   }
-  grid::upViewport()
 
-  # draw legend
-  if (do_legend) {
-    grid::downViewport("legend.vp")
-    grid::grid.draw(legend_grob)
-    grid::upViewport()
-  }
+  euler_grob$vp <- canvas_vp
 
   # draw strips
   if (do_strip_top) {
-    grid::downViewport("strip.top.vp")
+    strip_top_grob_children <- grid::gList()
     for (i in seq_len(layout[2L])) {
-      grid::pushViewport(grid::viewport(layout.pos.row = 1L,
-                                        layout.pos.col = i))
-      grid::grid.text(levels(strips$groups[[1L]])[i],
-                      just = "bottom",
-                      name = "strip.top.grob",
-                      gp = do.call(grid::gpar, strips$gp[i]))
-      grid::upViewport()
+      strip_top_grob_children[[i]] <- grid::textGrob(
+        levels(strips$groups[[1L]])[i],
+        just = "bottom",
+        name = paste0("strip.top.grob.", i),
+        gp = do.call(grid::gpar, strips$gp[i]),
+        vp = grid::viewport(layout.pos.row = 1L,
+                            layout.pos.col = i,
+                            name = paste0("strip.top.vp", i))
+      )
     }
-    grid::upViewport()
+
+    strip_top_grob <- grid::gTree(grid::nullGrob(),
+                                  children = strip_top_grob_children,
+                                  vp = strip_top_vp)
   }
 
   if (do_strip_left) {
-    grid::downViewport("strip.left.vp")
+    strip_left_grob_children <- grid::gList()
     for (i in seq_len(layout[1L])) {
-      grid::pushViewport(grid::viewport(layout.pos.row = i,
-                                        layout.pos.col = 1))
-      grid::grid.text(levels(strips$groups[[2L]])[i],
-                      just = "bottom",
-                      rot = 90,
-                      name = "strip.left.grob",
-                      gp = do.call(grid::gpar, strips$gp[i + layout[1L]]))
-      grid::upViewport()
+      strip_left_grob_children[[i]] <- grid::textGrob(
+        levels(strips$groups[[2L]])[i],
+        just = "bottom",
+        rot = 90,
+        name = paste0("strip.left.grob.", i),
+        gp = do.call(grid::gpar, strips$gp[i + layout[1L]]),
+        vp = grid::viewport(layout.pos.row = i,
+                            layout.pos.col = 1,
+                            name = paste0("strip.left.vp", i))
+      )
     }
-    grid::upViewport()
+    strip_left_grob <- grid::gTree(grid::nullGrob(),
+                                   children = strip_left_grob_children,
+                                   vp = strip_left_vp)
   }
-  grid::upViewport()
-  invisible(x)
 
-  # TODO: return a grob or gTree with suitable methods
+  # return a gTree object
+  grid::grobTree(
+    if (do_strip_top) strip_top_grob = strip_top_grob,
+    if (do_strip_left) strip_left_grob = strip_left_grob,
+    if (do_legend) legend_grob = legend_grob,
+    euler_grob = euler_grob,
+    vp = grid::viewport(layout = grid::grid.layout(nrow = nrow,
+                                                   ncol = ncol,
+                                                   widths = widths,
+                                                   heights = heights,
+                                                   respect = TRUE),
+                        name = "euler.vp"),
+    cl = "eulergram",
+    name = "euler.diagram"
+  )
 }
 
-#' @rdname plot.eulergram
-#' @export
-print.eulergram <- function(x, ...) {
-  graphics::plot(x, ...)
+#' Compute geometries and label locations
+#'
+#' @param x an object of class 'euler'
+#' @param do_fills do fills?
+#' @param do_edges do edges?
+#' @param do_labels do labels?
+#' @param do_quantities do quantities?
+#'
+#' @return a list object with slots for the various objects
+#' @keywords internal
+setup_geometry <- function(x,
+                           fills,
+                           edges,
+                           labels,
+                           quantities,
+                           n,
+                           id) {
+  dd <- x$ellipses
+  empty_sets <- is.na(dd[, 1L])
+  empty_subsets <- rowSums(id[, empty_sets, drop = FALSE]) > 0
+
+  orig <- x$original.values[!empty_subsets]
+  fitted <- x$fitted.values[!empty_subsets]
+  dd <- dd[!empty_sets, , drop = FALSE]
+
+  do_fills <- !is.null(fills)
+  do_edges <- !is.null(edges)
+  do_labels <- !is.null(labels)
+  do_quantities <- !is.null(quantities)
+
+  id <- id[!empty_subsets, !empty_sets, drop = FALSE]
+
+  h <- dd$h
+  k <- dd$k
+  a <- dd$a
+  b <- dd$b
+  phi <- dd$phi
+
+  n_e <- NROW(dd)
+  n_id <- 2L^n_e - 1L
+
+  e <- ellipse(h, k, a, b, phi, n)
+  e_x <- c(lapply(e, "[[", "x"), recursive = TRUE)
+  e_y <- c(lapply(e, "[[", "y"), recursive = TRUE)
+
+  limits <- get_bounding_box(h, k, a, b, phi)
+
+  # setup edges
+  if (do_edges)
+    edges <- list(x = e_x, y = e_y, id.lengths = rep.int(n, n_e))
+
+  if (do_fills) {
+    # overlay ellipses on top of each other
+    pieces <- fills <- vector("list", n_id)
+    for (i in rev(seq_len(n_id))) {
+      idx <- which(id[i, ])
+      n_idx <- length(idx)
+      if (n_idx == 1L) {
+        pieces[[i]] <- e[[idx[1]]]
+      } else {
+        pieces[[i]] <- poly_clip(e[[idx[1L]]], e[[idx[2L]]], "intersection")
+        if (n_idx > 2L) {
+          for (j in 3L:n_idx) {
+            pieces[[i]] <- poly_clip(pieces[[i]], e[[idx[j]]], "intersection")
+          }
+        }
+      }
+      for (ii in which(!id[i, ])) {
+        pieces[[i]] <- poly_clip(pieces[[i]], e[[ii]], "minus")
+      }
+
+      for (i in seq_along(pieces)) {
+        if (is.null(pieces[[i]]$x)) {
+          x0 <- lapply(pieces[[i]], "[[", "x")
+          y0 <- lapply(pieces[[i]], "[[", "y")
+        } else {
+          x0 <- pieces[[i]]$x
+          y0 <- pieces[[i]]$y
+        }
+        if (length(x0) > 0L) {
+          fills[[i]]$x <- c(x0, recursive = TRUE)
+          fills[[i]]$y <- c(y0, recursive = TRUE)
+          fills[[i]]$id.lengths <- lengths(x0)
+        }
+      }
+    }
+  }
+
+  if (do_labels || do_quantities) {
+    singles <- rowSums(id) == 1
+    empty <- abs(fitted) < sqrt(.Machine$double.eps)
+
+    centers <- cbind(t(locate_centers(h, k, a, b, phi, fitted)), seq_len(n_id))
+    rownames(centers) <- names(orig)
+
+    if (do_labels) {
+      labels <- list(labels = labels$labels[which(!empty_sets)])
+      center_labels <- labels$labels[!is.nan(centers[singles, 1L])]
+    }
+
+    labels_centers <- centers[!is.nan(centers[, 1L]) & singles, , drop = FALSE]
+
+    droprows <- rep.int(TRUE, NROW(centers))
+    for (i in which(is.nan(centers[singles, 1L]))) {
+      pick <- id[, i] & !empty & !is.nan(centers[, 1L])
+      labels_centers <- rbind(labels_centers, centers[which(pick)[1L], ])
+      if (do_labels)
+        center_labels <- c(center_labels, labels$labels[i])
+      droprows[which(pick)[1L]] <- FALSE
+    }
+
+    if (do_quantities) {
+      quantities_centers <-
+        centers[!is.nan(centers[, 1L]) & !singles & droprows, , drop = FALSE]
+      quantities_centers <- rbind(quantities_centers, labels_centers)
+      if (!is.null(quantities$labels))
+        quantities <- list(
+          labels =
+            quantities$labels[which(!empty_subsets)][quantities_centers[, 3L]]
+        )
+      else
+        quantities <- list(labels = orig[quantities_centers[, 3L]])
+      quantities$x <- quantities_centers[, 1L]
+      quantities$y <- quantities_centers[, 2L]
+      # quantities$x[void_sets] <- NA
+      # quantities$y[void_sets] <- NA
+    }
+
+    if (do_labels) {
+      labels$x <- labels_centers[, 1L]
+      labels$y <- labels_centers[, 2L]
+      labels$labels <- center_labels
+      # labels$x[void_sets] <- NA
+      # labels$y[void_sets] <- NA
+    }
+  }
+
+  list(ellipses = dd,
+       fitted.values = fitted,
+       original.values = orig,
+       fills = fills,
+       edges = edges,
+       labels = labels,
+       quantities = quantities,
+       empty_sets = empty_sets,
+       empty_subsets = empty_subsets,
+       xlim = limits$xlim,
+       ylim = limits$ylim)
 }
 
+
 #' Grobify Euler objects
 #'
-#' @param x an `euler` object
+#' @param x geometry data
+#' @param fills fills params
+#' @param edges edges params
+#' @param labels labels params
+#' @param quantities quantities params
+#' @param number current diagram number
 #'
 #' @return A [grid::gList()] is returned.
 #' @keywords internal
-setup_grobs <- function(x, fills, edges, labels, quantities) {
+setup_grobs <- function(x,
+                        fills,
+                        edges,
+                        labels,
+                        quantities,
+                        number) {
   data_labels <- x$labels
   data_edges <- x$edges
   data_fills <- x$fills
   data_quantities <- x$quantities
   fitted <- x$fitted.values
+  empty_sets <- x$empty_sets
+  empty_subsets <- x$empty_subsets
 
   do_labels <- !is.null(data_labels)
   do_edges <- !is.null(data_edges)
   do_fills <- !is.null(data_fills)
   do_quantities <- !is.null(data_quantities)
 
-  n_e <- nrow(x$ellipses)
+  n_e <- NROW(x$ellipses)
   n_id <- 2L^n_e - 1L
   id <- bit_indexr(n_e)
 
@@ -814,7 +809,7 @@ setup_grobs <- function(x, fills, edges, labels, quantities) {
                                      id.lengths = data_edges$id.lengths,
                                      default.units = "native",
                                      name = "edges.grob",
-                                     gp = edges$gp)
+                                     gp = edges$gp[which(!empty_sets)])
   }
 
   # fills
@@ -825,14 +820,16 @@ setup_grobs <- function(x, fills, edges, labels, quantities) {
         data_fills[[1]]$y,
         default.units = "native",
         name = "fills.grob",
-        gp = fills$gp[1L]
+        gp = fills$gp[which(!empty_subsets)[1L]]
       ))
     } else {
       fills_grob <- vector("list", n_id)
       fill_id <- seq_len(n_id)
+      empty_cols <- colSums(id & fitted > 0) == 0
+      # skip <- rowSums(id[, empty_cols, drop = FALSE]) > 0
       empty <- fitted < sqrt(.Machine$double.eps)
-      for (i in 1:n_e) {
-        if (empty[i]) {
+      for (i in seq_len(n_e)) {
+        if (empty[i] && !empty_cols[i]) {
           idx <- id[i, ]
           n_idx <- sum(idx)
           sub_id <- rowSums(id[, idx, drop = FALSE]) == n_idx
@@ -855,7 +852,7 @@ setup_grobs <- function(x, fills, edges, labels, quantities) {
             id.lengths = data_fills[[i]]$id.lengths,
             default.units = "native",
             name = paste0("fills.grob.", i),
-            gp = fills$gp[fill_id[i]]
+            gp = fills$gp[which(!empty_subsets)][fill_id[i]]
           )
       }
       fills_grob <- do.call(grid::gList, fills_grob)
@@ -873,11 +870,11 @@ setup_grobs <- function(x, fills, edges, labels, quantities) {
       label = data_quantities$labels,
       x = data_quantities$x,
       y = data_quantities$y,
-      rot = quantities$rot,
+      rot = quantities$rot[which(!empty_subsets)],
       vjust = ifelse(lab_id & do_labels, 1, 0.5),
       name = "quantities.grob",
       default.units = "native",
-      gp = quantities$gp
+      gp = quantities$gp[which(!empty_subsets)]
     )
   }
 
@@ -887,17 +884,43 @@ setup_grobs <- function(x, fills, edges, labels, quantities) {
       label = data_labels$labels,
       x = data_labels$x,
       y = data_labels$y,
-      rot = labels$rot,
+      rot = labels$rot[which(!empty_sets)],
       vjust = if (do_quantities) -0.5 else 0.5,
       name = "labels.grob",
       default.units = "native",
-      gp = labels$gp
+      gp = labels$gp[!empty_sets]
     )
   }
 
-  out <- grid::grobTree(if (do_fills) fills_grob,
-                        if (do_edges) edges_grob,
-                        if (do_labels) labels_grob,
-                        if (do_quantities) quantities_grob,
-                        name = "diagram.grob")
+  grid::grobTree(if (do_fills) fills_grob,
+                 if (do_edges) edges_grob,
+                 if (do_labels) labels_grob,
+                 if (do_quantities) quantities_grob,
+                 name = paste0("diagram.grob.", number))
 }
+
+#' Print (plot) Euler diagram
+#'
+#' This function is responsible for the actual drawing of
+#' `'eulergram'` objects created through [plot.euler()]. [print.eulergram()]
+#' is an alias for [plot.eulergram()], which has been provided so that
+#' [plot.euler()] gets called automatically.
+#'
+#' @param x an object of class `'eulergram'`, usually the output of
+#'   [plot.euler()]
+#' @param newpage if `TRUE`, opens a new page via [grid.newpage()] to draw on
+#' @param ... ignored
+#' @return A plot is drawn on the current device using [grid::Grid()] graphics.
+#' @export
+plot.eulergram <- function(x, newpage = TRUE, ...) {
+  if (isTRUE(newpage))
+    grid::grid.newpage()
+  grid::grid.draw(x)
+}
+
+#' @rdname plot.eulergram
+#' @export
+print.eulergram <- function(x, ...) {
+  graphics::plot(x, ...)
+}
+
